#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <stddef.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <unistd.h>
#ifdef SWIG
%{
#include "fns.h"
#include "dat.h"
%}
#else
#include "fns.h"
#include "dat.h"
#endif
#define	nelem(x)	(sizeof(x)/sizeof((x)[0]))

typedef struct Msg Msg;

int	timeout;

void
scpy(char *dst, char *src, size_t n)
{
	strncpy(dst, src, n-1);
	dst[n-1] = '\0';
}


int
query(char *answer, size_t ans_siz,
      char *err, size_t err_siz,
      char *netif,
      char *dst)
{
	unsigned char src[6];
	Msg m;
	int n, fd, pid;
	unsigned char cnt = 0;

	pid = getpid() & 0xff;
	errno = 0;
	fd = dial(netif, err, err_siz);
	if (fd == -1)
		return 0;
	if (!getea(fd, netif, src)) {
		scpy(err, "could not get local ethernet address", err_siz);
		return 0;
	}

	memmove(m.d, dst, 6);
	memmove(m.s, src, 6);
	m.t[0] = 0xbc;
	m.t[1] = 3;
	m.tag[0] = pid;
	m.tag[1] = ++cnt;
	write(fd, &m, 60);
	timewait(7);
	for (;;) {
		n = read(fd, &m, sizeof m);
		if (n == -1) {
			if (errno == EINTR) {
				alarm(0);
				break;
			}
			scpy(err, "reading response", err_siz);
			return 0;
		}
		if (m.tag[0] == pid && m.tag[1] == cnt) {
			size_t n_copy = n - offsetof(Msg, data);
			if (n_copy > ans_siz)
				n_copy = ans_siz;
			memmove(answer, m.data, n_copy);
			break;
		}
	}

	if (timeout) {
		scpy(err, "query timeout", err_siz);
		return 0;
	}

	return 1;
}

void
catch(int sig)
{
	(void) sig;
	timeout = 1;
}

void
timewait(int secs)	/* arrange for a sig_alarm signal after `secs' seconds */
{
	struct sigaction sa;
	
	memset(&sa, 0, sizeof sa);
	sa.sa_handler = catch;
	sa.sa_flags = SA_RESETHAND;
	sigaction(SIGALRM, &sa, NULL);
	alarm(secs);
}
